;; Generated package description from dismal.el  -*- no-byte-compile: t -*-
(define-package "dismal" "1.5.0.20200601.133435" "Dis Mode Ain't Lotus: Spreadsheet program Emacs" '((cl-lib "0")) :url "https://elpa.gnu.org/packages/dismal.html" :authors '((nil . "David Fox, fox@cs.nyu.edu") (nil . "Frank E. Ritter, ritter@cs.cmu.edu")) :maintainer '("UnMaintainer" . "emacs-devel@gnu.org"))
